From: tip-bot for Waiman Long <tipbot@zytor.com>
To: linux-tip-commits@vger.kernel.org
Cc: linux-kernel@vger.kernel.org, hpa@zytor.com, mingo@kernel.org,
torvalds@linux-foundation.org, peterz@infradead.org,
riel@redhat.com, Waiman.Long@hp.com, tglx@linutronix.de,
fengguang.wu@intel.com, scott.norton@hp.com,
maarten.lankhorst@canonical.com
Subject: [tip:locking/core] locking/selftest: Support queued rwlock
Date: Wed, 13 Aug 2014 03:57:11 -0700 [thread overview]
Message-ID: <tip-ae17ea0ec7d8fa64fbb773a52b2df5ba4766bcb8@git.kernel.org> (raw)
In-Reply-To: <1407345722-61615-3-git-send-email-Waiman.Long@hp.com>
Commit-ID: ae17ea0ec7d8fa64fbb773a52b2df5ba4766bcb8
Gitweb: http://git.kernel.org/tip/ae17ea0ec7d8fa64fbb773a52b2df5ba4766bcb8
Author: Waiman Long <Waiman.Long@hp.com>
AuthorDate: Wed, 6 Aug 2014 13:22:02 -0400
Committer: Ingo Molnar <mingo@kernel.org>
CommitDate: Wed, 13 Aug 2014 10:33:35 +0200
locking/selftest: Support queued rwlock
The queued rwlock does not support the use of recursive read-lock in
the process context. With changes in the lockdep code to check and
disallow recursive read-lock, it is also necessary for the locking
selftest to be updated to change the process context recursive read
locking results from SUCCESS to FAILURE for rwlock.
Signed-off-by: Waiman Long <Waiman.Long@hp.com>
Signed-off-by: Peter Zijlstra <peterz@infradead.org>
Cc: Maarten Lankhorst <maarten.lankhorst@canonical.com>
Cc: Rik van Riel <riel@redhat.com>
Cc: Scott J Norton <scott.norton@hp.com>
Cc: Fengguang Wu <fengguang.wu@intel.com>
Cc: Linus Torvalds <torvalds@linux-foundation.org>
Link: http://lkml.kernel.org/r/1407345722-61615-3-git-send-email-Waiman.Long@hp.com
Signed-off-by: Ingo Molnar <mingo@kernel.org>
---
lib/locking-selftest.c | 56 +++++++++++++++++++++++++++++++++++++++++++-------
1 file changed, 49 insertions(+), 7 deletions(-)
diff --git a/lib/locking-selftest.c b/lib/locking-selftest.c
index 872a15a..62af709 100644
--- a/lib/locking-selftest.c
+++ b/lib/locking-selftest.c
@@ -267,19 +267,46 @@ GENERATE_TESTCASE(AA_rsem)
#undef E
/*
* Special-case for read-locking, they are
* allowed to recurse on the same lock class:
*/
static void rlock_AA1(void)
{
RL(X1);
- RL(X1); // this one should NOT fail
+ RL(X1); // this one should fail
}
static void rlock_AA1B(void)
{
RL(X1);
- RL(X2); // this one should NOT fail
+ RL(X2); // this one should fail
+}
+
+static void rlock_AHA1(void)
+{
+ RL(X1);
+ HARDIRQ_ENTER();
+ RL(X1); // this one should NOT fail
+ HARDIRQ_EXIT();
+}
+
+static void rlock_AHA1B(void)
+{
+ RL(X1);
+ HARDIRQ_ENTER();
+ RL(X2); // this one should NOT fail
+ HARDIRQ_EXIT();
+}
+
+static void rlock_ASAHA1(void)
+{
+ RL(X1);
+ SOFTIRQ_ENTER();
+ RL(X1); // this one should NOT fail
+ HARDIRQ_ENTER();
+ RL(X1); // this one should NOT fail
+ HARDIRQ_EXIT();
+ SOFTIRQ_EXIT();
}
static void rsem_AA1(void)
@@ -1069,7 +1096,7 @@ static inline void print_testname(const char *testname)
print_testname(desc); \
dotest(name##_spin, FAILURE, LOCKTYPE_SPIN); \
dotest(name##_wlock, FAILURE, LOCKTYPE_RWLOCK); \
- dotest(name##_rlock, SUCCESS, LOCKTYPE_RWLOCK); \
+ dotest(name##_rlock, FAILURE, LOCKTYPE_RWLOCK); \
dotest(name##_mutex, FAILURE, LOCKTYPE_MUTEX); \
dotest(name##_wsem, FAILURE, LOCKTYPE_RWSEM); \
dotest(name##_rsem, FAILURE, LOCKTYPE_RWSEM); \
@@ -1830,14 +1857,14 @@ void locking_selftest(void)
printk(" --------------------------------------------------------------------------\n");
print_testname("recursive read-lock");
printk(" |");
- dotest(rlock_AA1, SUCCESS, LOCKTYPE_RWLOCK);
+ dotest(rlock_AA1, FAILURE, LOCKTYPE_RWLOCK);
printk(" |");
dotest(rsem_AA1, FAILURE, LOCKTYPE_RWSEM);
printk("\n");
print_testname("recursive read-lock #2");
printk(" |");
- dotest(rlock_AA1B, SUCCESS, LOCKTYPE_RWLOCK);
+ dotest(rlock_AA1B, FAILURE, LOCKTYPE_RWLOCK);
printk(" |");
dotest(rsem_AA1B, FAILURE, LOCKTYPE_RWSEM);
printk("\n");
@@ -1856,6 +1883,21 @@ void locking_selftest(void)
dotest(rsem_AA3, FAILURE, LOCKTYPE_RWSEM);
printk("\n");
+ print_testname("recursive rlock with interrupt");
+ printk(" |");
+ dotest(rlock_AHA1, SUCCESS, LOCKTYPE_RWLOCK);
+ printk("\n");
+
+ print_testname("recursive rlock with interrupt #2");
+ printk(" |");
+ dotest(rlock_AHA1B, SUCCESS, LOCKTYPE_RWLOCK);
+ printk("\n");
+
+ print_testname("recursive rlock with interrupt #3");
+ printk(" |");
+ dotest(rlock_ASAHA1, SUCCESS, LOCKTYPE_RWLOCK);
+ printk("\n");
+
printk(" --------------------------------------------------------------------------\n");
/*
next prev parent reply other threads:[~2014-08-13 10:58 UTC|newest]
Thread overview: 7+ messages / expand[flat|nested] mbox.gz Atom feed top
2014-08-06 17:22 [PATCH v7 0/2] lockdep: add support for queued rwlock Waiman Long
2014-08-06 17:22 ` [PATCH v7 1/2] locking/lockdep: Restrict the use of recursive read_lock() with qrwlock Waiman Long
2014-08-13 10:56 ` [tip:locking/core] " tip-bot for Waiman Long
2014-08-06 17:22 ` [PATCH v7 2/2] locking/selftest: Support queued rwlock Waiman Long
2014-08-07 9:58 ` Peter Zijlstra
2014-08-13 10:57 ` tip-bot for Waiman Long [this message]
-- strict thread matches above, loose matches on Subject: below --
2014-06-26 17:39 [PATCH v5 2/2] locking-selftest: Support queue rwlock Waiman Long
2014-07-17 11:00 ` [tip:locking/core] locking/selftest: Support queued rwlock tip-bot for Waiman Long
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=tip-ae17ea0ec7d8fa64fbb773a52b2df5ba4766bcb8@git.kernel.org \
--to=tipbot@zytor.com \
--cc=Waiman.Long@hp.com \
--cc=fengguang.wu@intel.com \
--cc=hpa@zytor.com \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-tip-commits@vger.kernel.org \
--cc=maarten.lankhorst@canonical.com \
--cc=mingo@kernel.org \
--cc=peterz@infradead.org \
--cc=riel@redhat.com \
--cc=scott.norton@hp.com \
--cc=tglx@linutronix.de \
--cc=torvalds@linux-foundation.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -267,19 +267,46 @@ GENERATE_TESTCASE(AA_rsem)
#undef E
/*
- * Special-case for read-locking, they are
- * allowed to recurse on the same lock class:
+ * Special-case for read-locking, they are not allowed to
+ * recurse on the same lock class except under interrupt context:
*/
static void rlock_AA1(void)
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox